@@ -6,6 +6,7 @@ from typing import TYPE_CHECKING, Generic, List, TypeVar
 from sila2.client.utils import call_rpc_function
 from sila2.framework.fully_qualified_identifier import FullyQualifiedIdentifier
 from sila2.framework.metadata import Metadata
+from sila2.framework.pb2.custom_protocols import AffectedCallsMessage
 
 if TYPE_CHECKING:
     from sila2.client.client_feature import ClientFeature
@@ -58,7 +59,7 @@ class ClientMetadata(Generic[ItemType]):
         """
         param_msg = self._wrapped_metadata.get_affected_calls_parameters_message()
 
-        response_msg = call_rpc_function(
+        response_msg: AffectedCallsMessage = call_rpc_function(
             getattr(self._parent_feature._grpc_stub, f"Get_FCPAffectedByMetadata_{self._wrapped_metadata._identifier}"),
             param_msg,
             metadata=None,
@@ -66,4 +67,7 @@ class ClientMetadata(Generic[ItemType]):
             origin=None,
         )
 
-        return self._wrapped_metadata.to_affected_calls_list(response_msg)
+        return [
+            FullyQualifiedIdentifier(identifier_sila_string.value)
+            for identifier_sila_string in response_msg.AffectedCalls
+        ]
