@@ -14,7 +14,7 @@ if TYPE_CHECKING:
     from sila2.client import SilaClient
     from sila2.client.client_observable_command import ClientObservableCommand
     from sila2.client.subscription import Subscription
-    from sila2.pb2_stubs.SiLAFramework_pb2 import CommandExecutionUUID as SilaCommandExecutionUUID
+    from sila2.framework.pb2.SiLAFramework_pb2 import CommandExecutionUUID as SilaCommandExecutionUUID
 
 ResponseType = TypeVar("ResponseType", bound=NamedTuple)
 IntermediateResponseType = TypeVar("IntermediateResponseType", bound=NamedTuple)
