@@ -1,15 +1,25 @@
 from __future__ import annotations
 
+import ssl
+import warnings
 from concurrent.futures import ThreadPoolExecutor
 from typing import TYPE_CHECKING, Dict, Optional, Set, Type, TypeVar, Union
 from uuid import UUID
 
 import grpc
+from cryptography import x509
+from cryptography.hazmat._oid import ObjectIdentifier
+from cryptography.hazmat.backends import default_backend
+from cryptography.x509 import ExtensionNotFound
 
 from sila2.client.client_feature import ClientFeature
 from sila2.framework import DefinedExecutionError
 from sila2.framework.defined_execution_error_node import DefinedExecutionErrorNode
-from sila2.framework.fully_qualified_identifier import FullyQualifiedIdentifier
+from sila2.framework.fully_qualified_identifier import (
+    FullyQualifiedDefinedExecutionErrorIdentifier,
+    FullyQualifiedFeatureIdentifier,
+    FullyQualifiedIdentifier,
+)
 
 if TYPE_CHECKING:
     from sila2.features.silaservice import SiLAServiceClient
@@ -32,11 +42,13 @@ class SilaClient:
     _channel: grpc.Channel
     _features: Dict[str, ClientFeature]
     _children_by_fully_qualified_identifier: Dict[FullyQualifiedIdentifier, HasFullyQualifiedIdentifier]
-    _registered_defined_execution_error_classes: Dict[FullyQualifiedIdentifier, Type[DefinedExecutionError]]
+    _registered_defined_execution_error_classes: Dict[
+        FullyQualifiedDefinedExecutionErrorIdentifier, Type[DefinedExecutionError]
+    ]
     _task_executor: ThreadPoolExecutor
 
-    _expected_features: Set[FullyQualifiedIdentifier] = {
-        FullyQualifiedIdentifier("org.silastandard/core/SiLAService/v1"),
+    _expected_features: Set[FullyQualifiedFeatureIdentifier] = {
+        FullyQualifiedFeatureIdentifier("org.silastandard/core/SiLAService/v1"),
     }
 
     __address: str
@@ -83,6 +95,7 @@ class SilaClient:
         self.__port = port
 
         target = f"{self.__address}:{self.__port}"
+        expected_server_uuid: Optional[UUID] = None
         if any(par is not None for par in (root_certs, private_key, cert_chain)):
             if insecure:
                 raise ValueError("Cannot use certificate information with insecure connections")
@@ -90,6 +103,15 @@ class SilaClient:
             credentials = grpc.ssl_channel_credentials(
                 root_certificates=root_certs, private_key=private_key, certificate_chain=cert_chain
             )
+            cert = x509.load_pem_x509_certificate(
+                ssl.get_server_certificate((address, port)).encode("ascii"), backend=default_backend()
+            )
+            try:
+                server_uuid_extension = cert.extensions.get_extension_for_oid(ObjectIdentifier("1.3.6.1.4.1.58583"))
+                expected_server_uuid = UUID(server_uuid_extension.value.public_bytes().decode("ascii"))
+            except ExtensionNotFound:
+                warnings.warn("Server certificate is missing the extension 1.3.6.1.4.1.58583 to specify its UUID")
+
             self._channel = grpc.secure_channel(target, credentials=credentials)
         elif insecure:
             self._channel = grpc.insecure_channel(target)
@@ -103,17 +125,29 @@ class SilaClient:
         # import locally to prevent circular import
         from sila2.features.silaservice import SiLAServiceFeature, UnimplementedFeature
 
+        # add SiLAService feature
         self.__add_feature(SiLAServiceFeature._feature_definition)
         self._register_defined_execution_error_class(
             SiLAServiceFeature.defined_execution_errors["UnimplementedFeature"], UnimplementedFeature
         )
 
+        # verify UUID from certificate
+        if expected_server_uuid is not None:
+            if UUID(self.SiLAService.ServerUUID.get()) != expected_server_uuid:
+                raise RuntimeError("Server UUID does not match the UUID specified in the server certificate")
+
+        # add other features
         for feature_id in self.SiLAService.ImplementedFeatures.get():
             if feature_id == SiLAServiceFeature.fully_qualified_identifier:
                 continue
-            self.__add_feature(self.SiLAService.GetFeatureDefinition(feature_id).FeatureDefinition)
+            try:
+                self.__add_feature(self.SiLAService.GetFeatureDefinition(feature_id).FeatureDefinition)
+            except ValueError as err:
+                warnings.warn(f"Found invalid feature {feature_id!r}, ignoring it: {err}")
 
-        found_features: Set[FullyQualifiedIdentifier] = {f.fully_qualified_identifier for f in self._features.values()}
+        found_features: Set[FullyQualifiedFeatureIdentifier] = {
+            f.fully_qualified_identifier for f in self._features.values()
+        }
         if not self._expected_features.issubset(found_features):
             raise RuntimeError(
                 f"Server does not implement the following required features: {self._expected_features - found_features}"
