@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Optional
 
 from zeroconf import Zeroconf
 
@@ -16,8 +16,10 @@ class SilaServiceBroadcaster:
     def __init__(self):
         self.zc = Zeroconf()
 
-    def register_server(self, server: SilaServer, address: str, port: int) -> SilaServiceInfo:
-        service_info = SilaServiceInfo(server, address, port)
+    def register_server(
+        self, server: SilaServer, address: str, port: int, ca: Optional[bytes] = None
+    ) -> SilaServiceInfo:
+        service_info = SilaServiceInfo(server, address, port, ca)
         self.zc.register_service(service_info)
         return service_info
 
