@@ -1,9 +1,12 @@
 import itertools
+import ssl
 import warnings
 from typing import Dict, Optional, Type
 from uuid import UUID
 
 from cryptography import x509
+from cryptography.hazmat._oid import NameOID
+from cryptography.hazmat.backends import default_backend
 from zeroconf import ServiceBrowser, ServiceInfo, ServiceListener, Zeroconf
 
 from sila2.client.sila_client import SilaClient
@@ -65,6 +68,14 @@ class SilaServiceListener(ServiceListener):
             else:
                 ca = self.__get_ca_from_service_info(service_info)
                 if ca is not None:
+                    # SiLA servers using untrusted certificates must have a certificate with Common Name "SiLA2"
+                    cert = x509.load_pem_x509_certificate(
+                        ssl.get_server_certificate((ip_address, port)).encode("ascii"), backend=default_backend()
+                    )
+                    common_name = cert.subject.get_attributes_for_oid(NameOID.COMMON_NAME)[0].value
+                    if common_name != "SiLA2":
+                        raise RuntimeError("Certificates of untrusted servers must have the common name 'SiLA2'")
+
                     self.services[service_name] = self.client_cls(ip_address, port, root_certs=ca)
                     warnings.warn(
                         RuntimeWarning(
