@@ -3,7 +3,7 @@ from __future__ import annotations
 from typing import TYPE_CHECKING
 
 from sila2.framework.abc.named_data_node import NamedDataNode
-from sila2.framework.fully_qualified_identifier import FullyQualifiedIdentifier
+from sila2.framework.fully_qualified_identifier import FullyQualifiedCommandParameterIdentifier
 
 if TYPE_CHECKING:
     from sila2.framework.command.command import Command
@@ -12,7 +12,7 @@ if TYPE_CHECKING:
 class Parameter(NamedDataNode):
     """Represents a command parameter"""
 
-    fully_qualified_identifier: FullyQualifiedIdentifier
+    fully_qualified_identifier: FullyQualifiedCommandParameterIdentifier
     """Fully qualified parameter identifier"""
     parent_command: Command
 
@@ -22,6 +22,6 @@ class Parameter(NamedDataNode):
             parent_command.parent_feature,
             getattr(parent_command.parent_feature._pb2_module, f"{parent_command._identifier}_Parameters"),
         )
-        self.fully_qualified_identifier = FullyQualifiedIdentifier(
+        self.fully_qualified_identifier = FullyQualifiedCommandParameterIdentifier(
             f"{parent_command.fully_qualified_identifier}/Parameter/{self._identifier}"
         )
