@@ -11,7 +11,7 @@ if TYPE_CHECKING:
     from sila2.framework.feature import Feature
 
 
-class AllowedTypes(Constraint):
+class AllowedTypes(Constraint[SilaAnyType]):
     allowed_types: List[str]
 
     def __init__(self, allowed_types: List[str]):
