@@ -9,7 +9,7 @@ if TYPE_CHECKING:
     from sila2.framework.feature import Feature
 
 
-class MaximalElementCount(Constraint):
+class MaximalElementCount(Constraint[list]):
     max_elements: int
 
     def __init__(self, max_elements: int):
