@@ -9,7 +9,7 @@ if TYPE_CHECKING:
     from sila2.framework.feature import Feature
 
 
-class MinimalElementCount(Constraint):
+class MinimalElementCount(Constraint[list]):
     min_elements: int
 
     def __init__(self, min_elements: int):
