@@ -1,19 +1,14 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Optional, Type
+from typing import Optional
 
 from sila2.framework.abc.data_type import DataType
 from sila2.framework.abc.named_data_node import NamedDataNode
-
-if TYPE_CHECKING:
-    from sila2.pb2_stubs import SiLAFramework_pb2
-    from sila2.pb2_stubs.SiLAFramework_pb2 import Boolean as SilaBoolean
+from sila2.framework.pb2 import SiLAFramework_pb2
+from sila2.framework.pb2.SiLAFramework_pb2 import Boolean as SilaBoolean
 
 
-class Boolean(DataType):
-    native_type = bool
-    message_type: Type[SilaBoolean]
-
+class Boolean(DataType[SilaBoolean, bool]):
     def __init__(self, silaframework_pb2_module: SiLAFramework_pb2):
         self.message_type = silaframework_pb2_module.Boolean
 
