@@ -1,15 +1,14 @@
 from __future__ import annotations
 
 from datetime import date, tzinfo
-from typing import TYPE_CHECKING, NamedTuple, Optional, Type
+from typing import NamedTuple, Optional
 
 from sila2.framework.abc.data_type import DataType
 from sila2.framework.abc.named_data_node import NamedDataNode
 from sila2.framework.data_types.timezone import Timezone
-
-if TYPE_CHECKING:
-    from sila2.pb2_stubs import SiLAFramework_pb2
-    from sila2.pb2_stubs.SiLAFramework_pb2 import Date as SilaDate
+from sila2.framework.errors.validation_error import ValidationError
+from sila2.framework.pb2 import SiLAFramework_pb2
+from sila2.framework.pb2.SiLAFramework_pb2 import Date as SilaDate
 
 
 class SilaDateType(NamedTuple):
@@ -19,10 +18,7 @@ class SilaDateType(NamedTuple):
     """Timezone"""
 
 
-class Date(DataType):
-    native_type = SilaDateType
-    message_type: Type[SilaDate]
-
+class Date(DataType[SilaDate, SilaDateType]):
     def __init__(self, silaframework_pb2_module: SiLAFramework_pb2):
         self.message_type = silaframework_pb2_module.Date
         self.__timezone_field = Timezone(silaframework_pb2_module)
@@ -42,6 +38,8 @@ class Date(DataType):
     def to_native_type(
         self, message: SilaDate, toplevel_named_data_node: Optional[NamedDataNode] = None
     ) -> SilaDateType:
+        if not message.HasField("timezone"):
+            raise ValidationError("Date type is missing required field 'timezone'")
         return SilaDateType(
             date(day=message.day, month=message.month, year=message.year),
             self.__timezone_field.to_native_type(message.timezone),
