@@ -1,17 +1,15 @@
 from __future__ import annotations
 
 import re
-from typing import TYPE_CHECKING, Optional, Type
+from typing import Optional, Type
 
 from sila2.framework.abc.data_type import DataType
 from sila2.framework.abc.named_data_node import NamedDataNode
-
-if TYPE_CHECKING:
-    from sila2.pb2_stubs import SiLAFramework_pb2
-    from sila2.pb2_stubs.SiLAFramework_pb2 import Integer as SilaInteger
+from sila2.framework.pb2 import SiLAFramework_pb2
+from sila2.framework.pb2.SiLAFramework_pb2 import Integer as SilaInteger
 
 
-class Integer(DataType):
+class Integer(DataType[SilaInteger, int]):
     native_type = int
     message_type: Type[SilaInteger]
 
