@@ -1,4 +1,5 @@
 from sila2.server.feature_implementation_base import FeatureImplementationBase
+from sila2.server.metadata_dict import MetadataDict
 from sila2.server.metadata_interceptor import MetadataInterceptor
 from sila2.server.observables.observable_command_instance import (
     ObservableCommandInstance,
@@ -8,6 +9,7 @@ from sila2.server.sila_server import SilaServer
 
 __all__ = [
     "FeatureImplementationBase",
+    "MetadataDict",
     "MetadataInterceptor",
     "ObservableCommandInstance",
     "ObservableCommandInstanceWithIntermediateResponses",
