@@ -1,7 +1,7 @@
 import uuid
 from dataclasses import dataclass
 from datetime import datetime, timedelta
-from typing import Any, Dict, List, Tuple
+from typing import Dict, List, Tuple
 from uuid import UUID
 
 from sila2.features.authenticationservice import (
@@ -13,6 +13,7 @@ from sila2.features.authenticationservice import (
 )
 from sila2.framework import FullyQualifiedIdentifier
 from sila2.server import SilaServer
+from sila2.server.metadata_dict import MetadataDict
 
 
 @dataclass
@@ -39,7 +40,7 @@ class AuthenticationServiceImpl(AuthenticationServiceBase):
         RequestedServer: str,
         RequestedFeatures: List[str],
         *,
-        metadata: Dict[FullyQualifiedIdentifier, Any]
+        metadata: MetadataDict
     ) -> Login_Responses:
         if UUID(RequestedServer) != self.parent_server.server_uuid:
             raise AuthenticationFailed
@@ -57,7 +58,7 @@ class AuthenticationServiceImpl(AuthenticationServiceBase):
         self.active_tokens[token.token] = token
         return Login_Responses(str(token.token), int(token.remaining_lifetime.total_seconds()))
 
-    def Logout(self, AccessToken: str, *, metadata: Dict[FullyQualifiedIdentifier, Any]) -> Logout_Responses:
+    def Logout(self, AccessToken: str, *, metadata: MetadataDict) -> Logout_Responses:
         try:
             token = self.active_tokens.pop(UUID(AccessToken))
         except (KeyError, ValueError):  # unknown token or no UUID
@@ -66,3 +67,5 @@ class AuthenticationServiceImpl(AuthenticationServiceBase):
         if token.remaining_lifetime.total_seconds() <= 0:
             self.active_tokens.pop(token.token)
             raise InvalidAccessToken
+
+        return Logout_Responses()
