@@ -10,6 +10,7 @@ from sila2.features.authorizationservice import (
 from sila2.framework import Command, Feature, FullyQualifiedIdentifier, Property
 from sila2.server import MetadataInterceptor, SilaServer
 from sila2.server.default_feature_implementations.authenticationservice_impl import AccessToken
+from sila2.server.metadata_dict import MetadataDict
 
 
 class AuthorizationServiceImpl(AuthorizationServiceBase):
@@ -35,14 +36,11 @@ class AuthorizationServiceInterceptor(MetadataInterceptor):
     access_tokens: Dict[UUID, AccessToken]
 
     def __init__(self, parent_server: SilaServer):
-        self.accesstoken_fqi = AuthorizationServiceFeature["AccessToken"].fully_qualified_identifier
-        super().__init__([self.accesstoken_fqi])
+        super().__init__([AuthorizationServiceFeature["AccessToken"]])
         self.access_tokens = parent_server.feature_servicers["AuthenticationService"].implementation.active_tokens
 
-    def intercept(
-        self, parameters: Any, metadata: Dict[FullyQualifiedIdentifier, Any], target_call: FullyQualifiedIdentifier
-    ) -> None:
-        raw_token: str = metadata.pop(self.accesstoken_fqi)
+    def intercept(self, parameters: Any, metadata: MetadataDict, target_call: FullyQualifiedIdentifier) -> None:
+        raw_token: str = metadata[AuthorizationServiceFeature["AccessToken"]]
 
         try:
             token = self.access_tokens[UUID(raw_token)]
