@@ -26,15 +26,17 @@ from sila2.framework.abc.sila_error import SilaError
 from sila2.framework.defined_execution_error_node import DefinedExecutionErrorNode
 from sila2.framework.errors.invalid_command_execution_uuid import InvalidCommandExecutionUUID
 from sila2.framework.feature import Feature
-from sila2.framework.fully_qualified_identifier import FullyQualifiedIdentifier
+from sila2.framework.fully_qualified_identifier import FullyQualifiedIdentifier, FullyQualifiedMetadataIdentifier
+from sila2.framework.pb2.custom_protocols import AffectedCallsMessage
 from sila2.framework.property.observable_property import ObservableProperty
 from sila2.framework.utils import FullyQualifiedIdentifierRegex, raise_as_rpc_error
 from sila2.server.feature_implementation_base import FeatureImplementationBase
+from sila2.server.metadata_dict import MetadataDict
 from sila2.server.observables.observable_command_manager import ObservableCommandManager
 from sila2.server.observables.subscription_manager_thread import SubscriptionManagerThread
 
 if TYPE_CHECKING:
-    from sila2.pb2_stubs.SiLAFramework_pb2 import CommandExecutionUUID as SilaCommandExecutionUUID
+    from sila2.framework.pb2.SiLAFramework_pb2 import CommandExecutionUUID as SilaCommandExecutionUUID
     from sila2.server import ObservableCommandInstance
     from sila2.server.sila_server import SilaServer
 
@@ -115,7 +117,7 @@ class FeatureImplementationServicer:
         def wrapper(request: Message, context: ServicerContext) -> Message:
             self.logger.info(f"Request: unobservable command {command_id}")
             metadata = self.__extract_metadata(context, self.parent_server, cmd)
-            allowed_errors = self.__find_allowed_defined_execution_errors(self.parent_server, cmd, metadata.keys())
+            allowed_errors = self.__find_allowed_defined_execution_errors(cmd, metadata)
             params = self.__unpack_parameters(cmd, request, context)
 
             with raises_rpc_errors(context, self.logger, allowed_errors):
@@ -133,7 +135,7 @@ class FeatureImplementationServicer:
         def wrapper(request, context: ServicerContext):
             self.logger.info(f"Request: unobservable property {property_id}")
             metadata = self.__extract_metadata(context, self.parent_server, prop)
-            allowed_errors = self.__find_allowed_defined_execution_errors(self.parent_server, prop, metadata.keys())
+            allowed_errors = self.__find_allowed_defined_execution_errors(prop, metadata)
 
             with raises_rpc_errors(context, self.logger, allowed_errors):
                 self.__apply_metadata_interceptors(None, metadata, prop.fully_qualified_identifier)
@@ -163,7 +165,7 @@ class FeatureImplementationServicer:
                 self.logger.info(f"Implementation returned {response!r}")
                 return cmd.responses.to_message(response)
 
-            command_manager = ObservableCommandManager(self.parent_server, cmd, _func_to_execute, metadata.keys())
+            command_manager = ObservableCommandManager(self.parent_server, cmd, _func_to_execute, metadata)
             self.observable_command_managers[command_id][command_manager.command_execution_uuid] = command_manager
 
             return self.feature._pb2_module.SiLAFramework__pb2.CommandConfirmation(
@@ -210,9 +212,7 @@ class FeatureImplementationServicer:
         def wrapper(request: SilaCommandExecutionUUID, context: ServicerContext) -> Message:
             self.logger.info(f"Request: observable command result for {command_id}")
             manager = self.__get_observable_command_manager(command_id, UUID(request.value), context)
-            allowed_errors = self.__find_allowed_defined_execution_errors(
-                self.parent_server, cmd, manager.metadata_identifiers
-            )
+            allowed_errors = self.__find_allowed_defined_execution_errors(cmd, manager.metadata)
 
             with raises_rpc_errors(context, self.logger, allowed_errors):
                 responses = manager.get_responses()
@@ -230,7 +230,7 @@ class FeatureImplementationServicer:
         def wrapper(request: Message, context: ServicerContext) -> Iterator[Message]:
             self.logger.info(f"Request: observable property subscription for {property_id}")
             metadata = self.__extract_metadata(context, self.parent_server, prop)
-            allowed_errors = self.__find_allowed_defined_execution_errors(self.parent_server, prop, metadata.keys())
+            allowed_errors = self.__find_allowed_defined_execution_errors(prop, metadata)
 
             with raises_rpc_errors(context, self.logger, allowed_errors):
                 self.__apply_metadata_interceptors(None, metadata, prop.fully_qualified_identifier)
@@ -244,11 +244,13 @@ class FeatureImplementationServicer:
 
         return wrapper
 
-    def __get_fpc_affected_by_metadata_call(self, metadata_id: str) -> Callable[[Message, ServicerContext], Message]:
+    def __get_fpc_affected_by_metadata_call(
+        self, metadata_id: str
+    ) -> Callable[[Message, ServicerContext], AffectedCallsMessage]:
         metadata_node = self.feature.metadata_definitions[metadata_id]
         impl_func = getattr(self.implementation, f"get_calls_affected_by_{metadata_id}")
 
-        def wrapper(request: Message, context: ServicerContext) -> Message:
+        def wrapper(request: Message, context: ServicerContext) -> AffectedCallsMessage:
             self.logger.info(f"Request: Calls affected by {metadata_id}")
             with raises_rpc_errors(context, self.logger):
                 affected_calls = impl_func()
@@ -263,11 +265,9 @@ class FeatureImplementationServicer:
             for manager in managers.values():
                 manager.cancel_producer()
 
-    def __apply_metadata_interceptors(
-        self, parameters: Any, metadata: Dict[FullyQualifiedIdentifier, Any], target: FullyQualifiedIdentifier
-    ):
+    def __apply_metadata_interceptors(self, parameters: Any, metadata: MetadataDict, target: FullyQualifiedIdentifier):
         for interceptor in self.parent_server.metadata_interceptors:
-            if interceptor.affected_metadata.issubset(metadata.keys()):
+            if any(m.fully_qualified_identifier in interceptor.affected_metadata for m in metadata):
                 self.logger.info(f"Applying metadata interceptor {interceptor}")
                 interceptor.intercept(parameters, metadata, target)
 
@@ -281,26 +281,20 @@ class FeatureImplementationServicer:
 
     def __find_allowed_defined_execution_errors(
         self,
-        server: SilaServer,
-        origin: Optional[Union[Command, Property]] = None,
-        metadata_identifiers: Optional[Iterable[FullyQualifiedIdentifier]] = None,
+        origin: Union[Command, Property],
+        metadata: Iterable[Metadata],
     ) -> List[DefinedExecutionErrorNode]:
-        allowed_errors = []
-        if origin is not None:
-            allowed_errors.extend(origin.defined_execution_errors)
-        if metadata_identifiers is not None:
-            for metadata_id in metadata_identifiers:
-                allowed_errors.extend(
-                    server.children_by_fully_qualified_identifier[metadata_id].defined_execution_errors
-                )
+        allowed_errors: List[DefinedExecutionErrorNode] = origin.defined_execution_errors.copy()
+        for m in metadata:
+            allowed_errors.extend(m.defined_execution_errors)
         self.logger.debug(f"Allowed defined execution errors: {[err._identifier for err in allowed_errors]}")
         return allowed_errors
 
     def __extract_metadata(
         self, context: grpc.ServicerContext, server: SilaServer, origin: Union[Property, Command]
-    ) -> Dict[FullyQualifiedIdentifier, Any]:
+    ) -> MetadataDict:
         self.logger.debug("Extracting metadata")
-        expected_metadata: Set[FullyQualifiedIdentifier] = set()
+        expected_metadata: Set[FullyQualifiedMetadataIdentifier] = set()
         for feature_servicer in server.feature_servicers.values():
             for meta_id, meta in feature_servicer.feature.metadata_definitions.items():
                 raw_affected_calls: List[Union[Feature, Command, Property, FullyQualifiedIdentifier]] = getattr(
@@ -318,7 +312,7 @@ class FeatureImplementationServicer:
                     expected_metadata.add(meta.fully_qualified_identifier)
         self.logger.debug(f"Required metadata: {expected_metadata}")
 
-        received_metadata: Dict[FullyQualifiedIdentifier, Any] = {}
+        received_metadata: Dict[FullyQualifiedMetadataIdentifier, Any] = {}
         for key, value in context.invocation_metadata():
             if re.fullmatch(
                 f"sila/{FullyQualifiedIdentifierRegex.MetadataIdentifier}/bin",
@@ -347,7 +341,7 @@ class FeatureImplementationServicer:
                 )
             if expected_metadata:
                 self.logger.warning(f"Server expects metadata for the SiLAService call {origin._identifier}. Ignoring.")
-            return {}
+            return MetadataDict(parent_server=self.parent_server)
 
         for expected_meta_fqi in expected_metadata:
             if expected_meta_fqi not in received_metadata:
@@ -355,7 +349,7 @@ class FeatureImplementationServicer:
                     InvalidMetadata(f"Did not receive required metadata '{expected_meta_fqi}'"), context
                 )
 
-        return received_metadata
+        return MetadataDict(parent_server=self.parent_server, base_dict=received_metadata)
 
     def __raise_as_rpc_error(self, error: SilaError, context: grpc.ServicerContext) -> NoReturn:
         self.logger.error(f"Raising exception as RpcError: {error!r}")
@@ -390,13 +384,21 @@ def raises_rpc_errors(
     try:
         yield
     except Exception as ex:
-        logger.exception("Caught exception, raising as RpcError")
+        logger.error(f"Caught exception {ex!r}, raising as RpcError")
         if isinstance(ex, NotImplementedError):
             context.abort(grpc.StatusCode.UNIMPLEMENTED, "The requested functionality is not implemented")
 
         if not isinstance(ex, SilaError):
             raise_as_rpc_error(UndefinedExecutionError.from_exception(ex), context)
 
+        if isinstance(ex, ValidationError) and ex.parameter_fully_qualified_identifier is None:
+            raise_as_rpc_error(
+                UndefinedExecutionError(
+                    f"Server tried to respond with invalid value: Caught ValidationError {str(ex)!r}"
+                ),
+                context,
+            )
+
         allowed_error_identifiers = [err.fully_qualified_identifier for err in allowed_defined_execution_errors]
         if isinstance(ex, DefinedExecutionError) and ex.fully_qualified_identifier not in allowed_error_identifiers:
             raise_as_rpc_error(UndefinedExecutionError.from_exception(ex), context)
