@@ -1,29 +1,34 @@
 from abc import ABC, abstractmethod
-from typing import Any, Dict, Iterable
+from typing import Any, Iterable
 
+from sila2.framework import Metadata
 from sila2.framework.fully_qualified_identifier import FullyQualifiedIdentifier
+from sila2.server.metadata_dict import MetadataDict
 
 
 class MetadataInterceptor(ABC):
-    def __init__(self, affected_metadata: Iterable[FullyQualifiedIdentifier]) -> None:
+    def __init__(self, affected_metadata: Iterable[Metadata]) -> None:
         """
         Intercepts SiLA Client Metadata
 
         Parameters
         ----------
         affected_metadata
-            Fully qualified identifiers of SiLA Client Metadata to intercept
+            SiLA Client Metadata to intercept
 
         Notes
         -----
         When a SiLA Client sends SiLA Client Metadata along with a command execution or property access request,
         the ``intercept()`` method of all affected metadata interceptors are called.
         """
-        self.affected_metadata = frozenset(affected_metadata)
+        self.affected_metadata = frozenset(m.fully_qualified_identifier for m in affected_metadata)
 
     @abstractmethod
     def intercept(
-        self, parameters: Any, metadata: Dict[FullyQualifiedIdentifier, Any], target_call: FullyQualifiedIdentifier
+        self,
+        parameters: Any,
+        metadata: MetadataDict,
+        target_call: FullyQualifiedIdentifier,
     ) -> None:
         """
         Called by the SiLA Server when receiving SiLA Client Metadata that is affected by this interceptor
