@@ -4,14 +4,14 @@ import datetime
 import uuid
 from concurrent.futures import Future
 from queue import Queue
-from typing import TYPE_CHECKING, Callable, Iterable
+from typing import TYPE_CHECKING, Callable
 
 from google.protobuf.message import Message
 
 from sila2.framework.command.execution_info import CommandExecutionInfo, CommandExecutionStatus, ExecutionInfo
 from sila2.framework.command.observable_command import ObservableCommand
 from sila2.framework.errors.command_execution_not_finished import CommandExecutionNotFinished
-from sila2.framework.fully_qualified_identifier import FullyQualifiedIdentifier
+from sila2.server.metadata_dict import MetadataDict
 from sila2.server.observables.observable_command_instance import (
     ObservableCommandInstance,
     ObservableCommandInstanceWithIntermediateResponses,
@@ -22,7 +22,7 @@ from sila2.server.observables.subscription_manager_thread import SubscriptionMan
 if TYPE_CHECKING:
     from grpc import ServicerContext
 
-    from sila2.pb2_stubs.SiLAFramework_pb2 import ExecutionInfo as SilaExecutionInfo
+    from sila2.framework.pb2.SiLAFramework_pb2 import ExecutionInfo as SilaExecutionInfo
     from sila2.server.sila_server import SilaServer
 
 
@@ -34,12 +34,12 @@ class ObservableCommandManager:
         parent_server: SilaServer,
         wrapped_command: ObservableCommand,
         impl_func: Callable[[ObservableCommandInstance], Message],
-        metadata_identifiers: Iterable[FullyQualifiedIdentifier],
+        metadata: MetadataDict,
     ) -> None:
         self.parent_server = parent_server
         self.command_execution_uuid = uuid.uuid4()
         self.wrapped_command = wrapped_command
-        self.metadata_identifiers = list(metadata_identifiers)
+        self.metadata = metadata
 
         # prepare subscriptions
         self.execution_info_queue = Queue()
