@@ -3,9 +3,9 @@ from __future__ import annotations
 import logging
 from queue import Queue
 from threading import Thread
-from typing import TYPE_CHECKING, Callable, Generic, List, Optional, TypeVar
+from typing import TYPE_CHECKING, Callable, Generic, List, Optional, TypeVar, Union
 
-from sila2.framework.fully_qualified_identifier import FullyQualifiedIdentifier
+from sila2.framework.fully_qualified_identifier import FullyQualifiedCommandIdentifier, FullyQualifiedPropertyIdentifier
 from sila2.server.observables.stream import Stream
 
 if TYPE_CHECKING:
@@ -20,7 +20,12 @@ class SubscriptionManagerThread(Thread, Generic[T]):
     __converter_func: Callable
     __last_item: Optional[T]
 
-    def __init__(self, origin_identifier: FullyQualifiedIdentifier, producer_queue: Queue[T], converter_func: Callable):
+    def __init__(
+        self,
+        origin_identifier: Union[FullyQualifiedPropertyIdentifier, FullyQualifiedCommandIdentifier],
+        producer_queue: Queue[T],
+        converter_func: Callable,
+    ):
         super().__init__(name=f"{self.__class__.__name__}-{origin_identifier}")
         self.__producer_stream = Stream.from_queue(producer_queue)
         self.__subscriber_streams = []
